using System.Collections.Generic;
using System.Linq;
using System.Windows;
using SciChart.Charting.ChartModifiers;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.RenderableSeries;
using SciChart.Core.Extensions;
using SciChart.Core.Utility.Mouse;

namespace SciChart.Examples.Examples.CreateSimpleChart
{
    public class HoverableSelectableChartModifier : ChartModifierBase
    {
        private HoverablePointMetadata _hoveredPoint;
        private IList<IPointMetadata> _selectedPoints = new List<IPointMetadata>();

        public override void OnModifierMouseUp(ModifierMouseArgs e)
        {
            base.OnModifierMouseUp(e);

            var hitTestInfo = HitTest(e.MousePoint);
            if (hitTestInfo?.Metadata != null)
            {
                // Change Selected state
                hitTestInfo.Value.Metadata.IsSelected = !hitTestInfo.Value.Metadata.IsSelected;

                // Store selected points
                if (hitTestInfo.Value.Metadata.IsSelected)
                {
                    _selectedPoints.Add(hitTestInfo.Value.Metadata);
                }
                else
                {
                    _selectedPoints.Remove(hitTestInfo.Value.Metadata);
                }
            }
            else
            {
                // Clear any previously selected points
                _selectedPoints.ForEachDo(p => p.IsSelected = false);
                _selectedPoints.Clear();
            }

            // Trigger a redraw
            ParentSurface.InvalidateElement();
        }

        public override void OnModifierMouseMove(ModifierMouseArgs e)
        {
            bool requiresRedraw = false;

            // Hit-test the series
            var hitTestInfo = HitTest(e.MousePoint);

            // Change Hovered state, if any series is hit
            var hoveredMetadata = hitTestInfo?.Metadata as HoverablePointMetadata;
            if (hoveredMetadata is { IsHovered: false })
            {
                hoveredMetadata.IsHovered = true;
                requiresRedraw = true;
            }

            // Clear any previous hovered state
            if (!ReferenceEquals(hoveredMetadata, _hoveredPoint) &&
                _hoveredPoint != null)
            {
                _hoveredPoint.IsHovered = false;
                requiresRedraw = true;
            }
            _hoveredPoint = hoveredMetadata;

            // Trigger a redraw
            if (requiresRedraw)
            {
                ParentSurface.InvalidateElement();
            }
        }

        private HitTestInfo? HitTest(Point atPoint)
        {
            // Translates the mouse point (from root grid coords) to ModifierSurface coords
            var translatedPoint = GetPointRelativeTo(atPoint, ModifierSurface);

            var hitTestInfo = ParentSurface
                .RenderableSeries
                .Select(s => s.HitTestProvider.HitTest(translatedPoint, 0d))
                .FirstOrDefault(hti => hti.IsHit);

            return hitTestInfo;
        }
    }
}